library ieee;
use ieee.std_logic_1164.all;

entity sata_controller is
  port (
    clk_62m5_i    :     std_logic;
    rst_62m5_n_i  :     std_logic;
    tx_clk_i      :     std_logic;
    rx_clk_i      :     std_logic;

    rdy_i         : in  std_logic;

    --  TX
    reg_tx_data_i : in  std_logic_vector(7 downto 0);
    reg_tx_wr_i   : in  std_logic;
    tx_data_o     : out std_logic_vector(7 downto 0);
    tx_k_o        : out std_logic;

    --  RX
    reg_rx_data_o : out std_logic_vector(7 downto 0);
    reg_rx_rd_i   : in  std_logic;
    reg_rx_o      : out std_logic;
    rx_data_i     : in  std_logic_vector(7 downto 0);
    rx_k_i        : in  std_logic
  );
end sata_controller;

architecture arch of sata_controller is
  signal tx_data_reg : std_logic_vector(7 downto 0);
  signal tx_data_wr : std_logic;

  signal rx_wr, rx_valid : std_logic;
begin
  inst_sync_tx: entity work.gc_sync_word_wr
    generic map (
      g_auto_wr => False,
      g_width => 8
    )
    port map (
      clk_in_i => clk_62m5_i,
      rst_in_n_i => rst_62m5_n_i,
      clk_out_i => tx_clk_i,
      rst_out_n_i => '1',
      data_i => reg_tx_data_i,
      wr_i => reg_tx_wr_i,
      busy_o => open,
      ack_o => open,
      data_o => tx_data_reg,
      wr_o => tx_data_wr
    );
  
  tx_data_o <= tx_data_reg when tx_data_wr = '1' else x"bc";
  tx_k_o <= not tx_data_wr;

  rx_valid <= (not rx_k_i) and rdy_i;

  inst_rx_data: entity work.gc_sync_word_wr
    generic map (
      g_auto_wr => False,
      g_width => 8
    )
    port map (
      clk_in_i => rx_clk_i,
      rst_in_n_i => '1',
      clk_out_i => clk_62m5_i,
      rst_out_n_i => rst_62m5_n_i,
      data_i => rx_data_i,
      wr_i => rx_valid,
      busy_o => open,
      ack_o => open,
      data_o => reg_rx_data_o,
      wr_o => rx_wr
    );

  process(clk_62m5_i) is
  begin
    if rising_edge(clk_62m5_i) then
      if rst_62m5_n_i = '0' then
        reg_rx_o <= '0';
      else
        if rx_wr = '1' then
          reg_rx_o <= '1';
        elsif reg_rx_rd_i = '1' then
          reg_rx_o <= '0';
        end if;
      end if;
    end if;
  end process;
end arch;